import 'package:flutter/material.dart';
import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:wordguessing/models/activity/activity.dart';
import 'package:wordguessing/models/settings/settings_activity.dart';
import 'package:wordguessing/models/settings/settings_global.dart';

part 'activity_state.dart';

class ActivityCubit extends HydratedCubit<ActivityState> {
  ActivityCubit()
      : super(ActivityState(
          currentActivity: Activity.createEmpty(),
        ));

  void updateState(Activity activity) {
    emit(ActivityState(
      currentActivity: activity,
    ));
  }

  void refresh() {
    final Activity activity = Activity(
      // Settings
      activitySettings: state.currentActivity.activitySettings,
      globalSettings: state.currentActivity.globalSettings,
      // State
      isRunning: state.currentActivity.isRunning,
      isStarted: state.currentActivity.isStarted,
      isFinished: state.currentActivity.isFinished,
      animationInProgress: state.currentActivity.animationInProgress,
      // Base data
      word: state.currentActivity.word,
      otherWords: state.currentActivity.otherWords,
      images: state.currentActivity.images,
      // Game data
      recentWordsKeys: state.currentActivity.recentWordsKeys,
      questionsCount: state.currentActivity.questionsCount,
      goodAnswers: state.currentActivity.goodAnswers,
      wrongAnswers: state.currentActivity.wrongAnswers,
    );
    // game.dump();

    updateState(activity);
  }

  void startNewActivity({
    required ActivitySettings activitySettings,
    required GlobalSettings globalSettings,
  }) {
    final Activity newActivity = Activity.createNew(
      // Settings
      activitySettings: activitySettings,
      globalSettings: globalSettings,
    );

    updateState(newActivity);
    nextWord();

    newActivity.dump();

    updateState(newActivity);
    refresh();
  }

  void quitActivity() {
    state.currentActivity.isRunning = false;
    refresh();
  }

  void resumeSavedActivity() {
    state.currentActivity.isRunning = true;
    refresh();
  }

  void deleteSavedActivity() {
    state.currentActivity.isRunning = false;
    state.currentActivity.isFinished = true;
    refresh();
  }

  void nextWord() {
    state.currentActivity.pickNewWord();
    refresh();
  }

  void checkWord(word) {
    if (state.currentActivity.word.key == word.key) {
      state.currentActivity.goodAnswers++;
      nextWord();
    } else {
      state.currentActivity.wrongAnswers++;
    }
    refresh();
  }

  @override
  ActivityState? fromJson(Map<String, dynamic> json) {
    final Activity currentActivity = json['currentActivity'] as Activity;

    return ActivityState(
      currentActivity: currentActivity,
    );
  }

  @override
  Map<String, dynamic>? toJson(ActivityState state) {
    return <String, dynamic>{
      'currentActivity': state.currentActivity.toJson(),
    };
  }
}
