import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:wordguessing/config/default_activity_settings.dart';

class ActivitySettings {
  final String gameType;
  final String lang;

  ActivitySettings({
    required this.gameType,
    required this.lang,
  });

  static String getGameTypeValueFromUnsafe(String gameType) {
    if (DefaultActivitySettings.allowedGameTypeValues.contains(gameType)) {
      return gameType;
    }

    return DefaultActivitySettings.defaultGameTypeValue;
  }

  static String getLangValueFromUnsafe(String lang) {
    if (DefaultActivitySettings.allowedLangValues.contains(lang)) {
      return lang;
    }

    return DefaultActivitySettings.defaultLangValue;
  }

  factory ActivitySettings.createDefault() {
    return ActivitySettings(
      gameType: DefaultActivitySettings.defaultGameTypeValue,
      lang: DefaultActivitySettings.defaultLangValue,
    );
  }

  void dump() {
    printlog('$ActivitySettings:');
    printlog('  ${DefaultActivitySettings.parameterCodeGameType}: $gameType');
    printlog('  ${DefaultActivitySettings.parameterCodeLangValue}: $lang');
    printlog('');
  }

  @override
  String toString() {
    return '$ActivitySettings(${toJson()})';
  }

  Map<String, dynamic>? toJson() {
    return <String, dynamic>{
      DefaultActivitySettings.parameterCodeGameType: gameType,
      DefaultActivitySettings.parameterCodeLangValue: lang,
    };
  }
}
